# fichier bash à sourcer
# incompatible sh /dash !!!
# fonctions utiles pour générer les configurations des services sambaedu
#
# Denis Bonnenfant et al.
#
# licence GPL

#Couleurs
COLTITRE="\033[1;35m"       # Rose
COLDEFAUT="\033[0;33m"      # Brun-jaune
COLCMD="\033[1;37m\c"       # Blanc
COLERREUR="\033[1;31m"      # Rouge
COLTXT="\033[0;37m"         # Gris
COLINFO="\033[0;36m"        # Cyan
COLPARTIE="\033[1;34m\c"    # Bleu

cdr2mask() {
    # Number of args to shift, 255..255, first non-255 byte, zeroes
    set -- $(( 5 - ($1 / 8) )) 255 255 255 255 $(( (255 << (8 - ($1 % 8))) & 255 )) 0 0 0
    [ $1 -gt 1 ] && shift $1 || shift
    echo ${1-0}.${2-0}.${3-0}.${4-0}
}

# Fonction permettant de récuperer la configuration réseau complète
my_network() {
    IFS=" "
    read my_gateway my_interface<<<$(ip -o -f inet route show default 0.0.0.0/0 | cut -d ' ' -f3,5)
    read my_address my_cdr my_broadcast<<<$(ip -o -f inet addr show dev "$my_interface" | awk '{sub("/", " ", $4); print $4, $6}')
    my_mask=$(cdr2mask $my_cdr)
    IFS=
    my_network=$(ip -o -f inet route show dev $my_interface src $my_address | cut -d/ -f1)
    my_hostname=$(hostname -s 2>/dev/null)
    #my_domain=$(hostname -d 2>/dev/null) || true
    my_fqdn=$(hostname -f 2>/dev/null) || true
    # attention ne donne pas le dns externe si ad est configuré
    my_dnsserver=$(grep -m 1 "^nameserver" /etc/resolv.conf | cut -d" " -f2) || true
    my_domain=$(grep -m 1 -e "^domain\s*.*$" -o -e "^search\s*.*$" /etc/resolv.conf | cut -d" " -f2) || true
    my_domain=${my_domain,,}
    my_proxy="$http_proxy"
    read my_proxy_address my_proxy_port<<<$(echo $my_proxy | sed "s!http://!!" | cut -d ':' -f1,2)
    if [ -e "etc/sambaedu/sambaedu.conf.d/dhcp.conf" ]; then
        my_vlan=$(grep $my_network /etc/sambaedu/sambaedu.conf.d/dhcp.conf | cut -d= -f1 | sed "s/^.*_//")
    fi
    filtre="^\s*iface\s*${my_interface}\s*inet.*$"
    my_mode=$(grep -e "$filtre" /etc/network/interfaces | cut -d" " -f4)
}

function check_proxy_params() {
    if [ -n "$config_proxy_address" ] && [ -z "$config_proxy_port" ]; then
        echo "Paramètres incorrect"
        exit 1
    fi
}

function clean_old_proxy_params() {
    profile_file="/etc/profile"
    wgetrc_file="/etc/wgetrc"
    sed -i '/no_proxy=/d' $profile_file
    sed -i '/http_proxy=/d' $profile_file
    sed -i '/https_proxy=/d' $profile_file
    sed -i '/ftp_proxy=/d' $profile_file
    sed -i '/^export http_proxy/d' $profile_file
    sed -i '/^no_proxy/d' $wgetrc_file
    sed -i '/^http_proxy/d' $wgetrc_file
    sed -i '/^https_proxy/d' $wgetrc_file
    sed -i '/^use_proxy = on/d' $wgetrc_file
    . $profile_file
    if [ -e /usr/sbin/NetworkManager -a "$config_proxy_type" != "automatique" ] ; then
        OLDIFS=$IFS
        IFS="
"
        for interface in $(nmcli -t -f UUID c show) ; do
            nmcli c modify $interface proxy.method none   || true
        done
        IFS=$OLDIFS
    fi
    # On désactive le cache utilisé par l'installeur dans la conf apt
    if [ -f "/etc/apt/apt.conf" ]; then
        rm -f /etc/apt/apt.conf || true
    fi
    if [ -f "/etc/apt/apt.conf.d/99proxy" ]; then
        rm -f /etc/apt/apt.conf.d/99proxy || true
    fi
    if [ -f /etc/apt/apt.conf.d/99proxy.conf ]; then
        rm -f /etc/apt/apt.conf.d/99proxy.conf
    fi
}

function set_proxy_params() {
    profile_file="/etc/profile"
    wgetrc_file="/etc/wgetrc"
    echo "no_proxy=\".${config_domain},${config_se4fs_name}\"" >> $profile_file
    echo "http_proxy=\"http://$1\"" >> $profile_file
    echo "https_proxy=\"http://$1\"" >> $profile_file
    echo "ftp_proxy=\"http://$1\"" >> $profile_file
    echo "export http_proxy https_proxy ftp_proxy no_proxy" >> $profile_file
    echo "no_proxy = \".${config_domain},${config_se4fs_name}\"" >> $wgetrc_file
    echo "http_proxy = http://$1" >> $wgetrc_file
    echo "https_proxy = http://$1" >> $wgetrc_file
    echo "use_proxy = on" >> $wgetrc_file
    . $profile_file
    if [ -n "$config_apt_proxy" ]; then
        apt_http_proxy="$config_apt_proxy"
    else
        apt_http_proxy="http://$1"
    fi
    echo "Acquire::http::proxy \"$apt_http_proxy\";
Acquire::https::proxy \"$apt_http_proxy\";" > /etc/apt/apt.conf.d/99proxy
}

function set_proxy_url() {
    if [ -e /usr/sbin/NetworkManager ]; then
        OLDIFS=$IFS
        IFS="
"
        for interface in $(nmcli -t -f UUID c show); do
            nmcli c modify $interface proxy.method auto  || true
        done
        for interface in $(nmcli -t -f UUID c show); do
            if nmcli c show $interface | grep "proxy\.method.*auto"; then
                nmcli c modify $interface proxy.pac-url "$1" || true
            fi
        done
        IFS=$OLDIFS
    fi
}

#Activation Debug
function debug() {
    debug="1"
    if [ "$debug" = "1" ]; then
        set -x
        poursuivre
    fi
}

# Affichage de la partie actuelle
function show_part() {
    echo -e "$COLPARTIE"
    echo -e "--------\r"
    echo -e "$1 \r"
    echo -e "-------- $COLTXT\r"
    # sleep 1
}

# Affichage d'une info
function show_info() {
    echo -e "$COLINFO\c"
    echo -e "$1 $COLTXT\r"
    # sleep 1
}

function show_progress() {
    [ $cpt -eq 1 ] && echo -e "Traitement de $cpt_fin éléments lancé...\r"
    echo -e "$cpt/$cpt_fin... \c"
    [ $cpt -eq $cpt_fin ] && echo -e "Terminé !\r"
    let cpt++
}

function check_winbind() {
    show_info "Vérification de Winbind."
    if systemctl status winbind.service | grep -q "running"; then
        echo -e "Winbind est bien actif.\r"
    else
        echo -e "Winbind n'est pas actif ou est en défaut - abandon !\r"
        echo -e "Vous pouvez tenter de le relancer par systemctl restart winbind.\r"
        exit 1
    fi
}

function correct_home() {
    show_part "Remise en place des droits sur /home ."
    check_winbind
    show_info "Lancement de la remise en place..."
    liste_users="$(wbinfo -u)"
    cpt=1
    cpt_fin="$(ls /home | wc -l)"
    chown root:domain\ users /home/profiles
    chmod 777 /home/profiles
    # chgrp domain\ users /home/profiles
    ls /home | while read useruid
    do
        show_progress
        if [ "$useruid" != "templates" -a "$useruid" != "profiles" -a "$useruid" != "trash" ]; then
            if echo "$liste_users" | grep -q "$useruid"
            then
                # pas d'acls sur le dossier
                setfacl -b -R /home/$useruid

                # remise en place des droits
                chmod -R 770 /home/$useruid
                chown -R $useruid:www-admin /home/$useruid
                # prop=`stat -c%G /home/$useruid`

                # droits du profile
                if [ -d "/home/profiles/${useruid}.V6/" ]; then
                    setfacl -b -R "/home/profiles/${useruid}.V6/"
                    chown -R $useruid:domain\ users "/home/profiles/${useruid}.V6/"
                    chmod -R 770 "/home/profiles/${useruid}.V6/"
                    setfacl -R -m user::rwx,group::---,group:NT\\040Authority\\\\system:rwx,group:domain\\040users:---,group:$useruid:rwx,mask::rwx,other::---,default:user::rwx,default:user:$useruid:rwx,default:group::---,default:group:NT\\040Authority\\\\system:rwx,default:group:domain\\040users:---,default:group:$useruid:rwx,default:mask::rwx,default:other::--- "/home/profiles/${useruid}.V6/"
                else
                    show_info "$useruid ne semble pas avoir de profile /home/profiles/$useruid sera ignoré"
                fi
            else
                show_info "$useruid ne semble pas faire partie des utilisateurs /home/$useruid sera ignoré"
            fi
        fi
    done
    show_info "Fin du traitement."
}

function correct_printers() {
    show_part "Remise en place des droits printers."
    # check_winbind
    show_info "Lancement de la remise en place..."
    setfacl -R -m user::rwx,user:www-admin:rwx,group:domain\\040users:r-x,group:domain\\040admins:rwx,group:domain\\040computers:r-x,mask::rwx,other::---,default:user::rwx,default:user:www-admin:rwx,default:group:domain\\040users:r-x,default:group:domain\\040admins:rwx,default:group:domain\\040computers:r-x,default:mask::rwx,default:other::--- /var/lib/samba/printers
    show_info "Fin du traitement."
}

function correct_wpkg() {
    show_part "Remise en place des droits pour wpkg."
    # check_winbind
    show_info "Lancement de la remise en place..."
    chown -R www-admin:root /var/sambaedu/unattended/install/wpkg
    chown -R www-admin:root /var/sambaedu/unattended/install/packages
    setfacl -R -m user::rwx,user:www-admin:rwx,user:admin:rwx,group::r-x,group:domain\\040admins:r-x,group:domain\\040computers:r-x,mask::rwx,other::---,default:user::rwx,default:user:www-admin:rwx,default:user:admin:rwx,default:group::r-x,default:group:domain\\040admins:r-x,default:group:domain\\040computers:r-x,default:mask::rwx,default:other::--- /var/sambaedu/unattended/install/wpkg
    setfacl -R -m user::rwx,user:www-admin:rwx,user:admin:rwx,group::r-x,group:domain\\040admins:r-x,group:domain\\040computers:r-x,mask::rwx,other::---,default:user::rwx,default:user:www-admin:rwx,default:user:admin:rwx,default:group::r-x,default:group:domain\\040admins:r-x,default:group:domain\\040computers:r-x,default:mask::rwx,default:other::--- /var/sambaedu/unattended/install/packages
    setfacl -R -m user::rwx,user:www-admin:rwx,user:admin:rwx,group::r-x,group:domain\\040admins:r-x,group:domain\\040computers:rwx,mask::rwx,other::---,default:user::rwx,default:user:www-admin:rwx,default:user:admin:rwx,default:group::r-x,default:group:domain\\040admins:r-x,default:group:domain\\040computers:rwx,default:mask::rwx,default:other::--- /var/sambaedu/unattended/install/wpkg/rapports
    show_info "Fin du traitement."
}

function correct_var_sambaedu() {
    show_part "Remise en place des droits pour /var/sambaedu/ ."
    # check_winbind
    show_info "Lancement de la remise en place..."
    mkdir -p /var/sambaedu/Docs
    chown admin:domain\ admins /var/sambaedu/Docs
    setfacl -m user::rwx,group::rwx,group:domain\\040admins:rwx,mask::rwx,other::r-x,default:user::rwx,default:group::rwx,default:group:domain\\040admins:rwx,default:mask::rwx,default:other::r-x /var/sambaedu/Docs
    mkdir -p /var/sambaedu/Classes
    chown www-admin:domain\ admins /var/sambaedu/Classes
    setfacl -m user::rwx,group::rwx,group:domain\\040admins:rwx,mask::rwx,other::r-x,default:user::rwx,default:group::rwx,default:group:domain\\040admins:rwx,default:mask::rwx,default:other::r-x /var/sambaedu/Classes
    mkdir -p /var/sambaedu/Progs
    chown admin:domain\ admins /var/sambaedu/Progs
    setfacl -m user::rwx,group::rwx,group:domain\\040admins:rwx,mask::rwx,other::r-x,default:user::rwx,default:group::rwx,default:group:domain\\040admins:rwx,default:mask::rwx,default:other::r-x /var/sambaedu/Progs
    mkdir -p /var/sambaedu/Progs/ro
    chown admin:domain\ admins /var/sambaedu/Progs/ro
    setfacl -m user::rwx,group::rwx,group:domain\\040admins:rwx,mask::rwx,other::r-x,default:user::rwx,default:group::rwx,default:group:domain\\040admins:rwx,default:mask::rwx,default:other::r-x /var/sambaedu/Progs/ro
    mkdir -p /var/sambaedu/Progs/rw
    chown admin:domain\ admins /var/sambaedu/Progs/rw
    setfacl -m user::rwx,group::rwx,group:domain\\040admins:rwx,mask::rwx,other::r-x,default:user::rwx,default:group::rwx,default:group:domain\\040admins:rwx,default:mask::rwx,default:other::rwx /var/sambaedu/Progs/rw
    show_info "Fin du traitement."
}

function dl_key() {
    url="$1"
}

function make_krb5.conf() {
    # correction valeur apparue par erreur.
    if [ "$config_se4ad_slave_ip" == "out;" ]; then
        set_config sambaedu se4ad_slave_ip
    fi
    if [ "$my_address" == "$config_se4ad_ip" -o "$my_address" == "$config_se4ad_etab_ip" -o "$my_address" == "$config_se4ad_slave_ip" ]; then
        # serveur AD
        echo "[libdefaults]
    default_realm = $config_REALM
    dns_lookup_realm = false
    dns_lookup_kdc = false
[realms]
    $config_REALM = {
    kdc = 127.0.0.1
    kdc = $config_se4ad_ip
    }
">/tmp/krb5.conf
    else
        # Serveur FS ou client
        echo "[libdefaults]
    default_realm = $config_REALM
    dns_lookup_realm = false
    dns_lookup_kdc = true
">/tmp/krb5.conf
    fi
    cp /tmp/krb5.conf /etc/krb5.conf
}

function make_apt_sources() {
    . /etc/os-release
    if [ -f  "/etc/apt/keyrings/apt-setup_local0.gpg" ]; then
        mv "/etc/apt/keyrings/apt-setup_local0.gpg" "/etc/apt/trusted.gpg.d/sambaedu.gpg"
    elif [ ! -f "/etc/apt/trusted.gpg.d/sambaedu.gpg" ]; then
        wget -O /etc/apt/trusted.gpg.d/sambaedu.gpg http://deb.sambaedu.org/debian/sambaedu.gpg.key || true
    fi
    canal=$(grep "deb.sambaedu.org" /etc/apt/sources.list | awk '{print $5}')
    if [ -n "$canal" ]; then
        sed -i "/^.*deb\.sambaedu\.org.*$/d" /etc/apt/sources.list
        echo "deb http://deb.sambaedu.org/debian $VERSION_CODENAME $canal">/etc/apt/sources.list.d/se4.list
    fi
}

function install_backports() {
    if [ "$1" == "bookworm" ]; then
#        if [ ! -f "/etc/apt/sources.list.d/backports.list" ]; then
#            echo "deb http://deb.debian.org/debian/ bookworm-backports main" > /etc/apt/sources.list.d/backports.list
            echo "deb http://deb.sambaedu.org/debian/ bookworm-backports main" > /etc/apt/sources.list.d/backports.list
#        fi
        if [ ! -f "/etc/apt/preferences.d/samba-backports" ]; then
            echo "Package: samba libldb2 libnss-winbind libsmbclient libtalloc2 libtdb1 libtevent0 libwbclient0 python3-ldb python3-samba python3-talloc python3-tdb samba-ad-provision samba-common samba-common-bin samba-dsdb-modules samba-libs samba-vfs-modules smbclient tdb-tools winbind
Pin: release n=bookworm-backports
Pin-Priority: 990" > /etc/apt/preferences.d/samba-backports
        fi
        sed -i "/bookworm-backports/d" /etc/apt/sources.list
        rm -f "/etc/apt/preferences.d/backports.pref" || true
    else
        sed -i "/bullseye-backports/d" /etc/apt/sources.list
        if [ -f "/etc/apt/sources.list.d/backports.list" ]; then
            rm -f "/etc/apt/sources.list.d/backports.list"
        fi
        if [ -f "/etc/apt/preferences.d/backports.pref" ]; then
            rm -f "/etc/apt/preferences.d/backports.pref"
        fi
    fi
}
